@@ -1,6 +1,8 @@
 #!/bin/sh
 
 test_description='handling of common mistakes people may make with submodules'
+
+TEST_PASSES_SANITIZE_LEAK=true
 . ./test-lib.sh
 
 test_expect_success 'create embedded repository' '
@@ -30,8 +32,7 @@ test_expect_success 'no warning when updating entry' '
 
 test_expect_success 'submodule add does not warn' '
 	test_when_finished "git rm -rf submodule .gitmodules" &&
-	git -c protocol.file.allow=always \
-		submodule add ./embed submodule 2>stderr &&
+	git submodule add ./embed submodule 2>stderr &&
 	test_i18ngrep ! warning stderr
 '
 
